# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the class object.

Used to create a thermal object and apply or remove fields.

"""

from .. import mats
import os
import copy
from .. import solvers


class object:

    """object class

    This class creates thermal objects to be used in more complex systems.
    It includes 2 methods to apply and remove fields.

    """

    def __init__(self, amb_temperature, materials=('Cu',), borders=(1, 11),
                 materials_order=(0,), dx=0.01, dt=0.1, file_name='data.txt',
                 boundaries=(0, 0), Q=[], Q0=[], initial_state=False,
                 heat_save=False):
        """Initializes the object.

        amb_temperature: ambient temperature of the whole system
        materials: list of strings of all the used materials present in the
            folder materials
        borders: list of the points where there is a change of material
        materials_order: list of the materials list indexes that defines the
            material properties given by borders
        dx: the space step
        dt: the times step
        file_name: file name where the temperature and heat flux are saved
        boundaries: list of two entries that define the boundary condition
            for tempreture. If 0 the boundary condition is insulation
        Q: list of fixed heat source coeficient.
        Q0: list of temperature dependent heat source coefficient.
        initial_state: initial state of the materials. True if applied field
            and False is removed field.
        heat_save: True if saving the heat at the two borders.

        """

        # check the validity of inputs
        cond01 = isinstance(amb_temperature, float)
        cond01 = cond01 or isinstance(amb_temperature, int)
        cond02 = isinstance(materials, tuple)
        cond03 = isinstance(borders, tuple)
        cond04 = isinstance(materials_order, tuple)
        cond05 = isinstance(dx, int) or isinstance(dx, float)
        cond06 = isinstance(dt, int) or isinstance(dt, float)
        cond07 = isinstance(file_name, unicode)
        cond07 = cond07 or isinstance(file_name, str)
        cond08 = isinstance(boundaries, tuple)
        cond09 = isinstance(Q, list)
        cond10 = isinstance(Q0, list)
        cond11 = isinstance(initial_state, bool)
        cond12 = isinstance(heat_save, bool)
        condition = cond01 and cond02 and cond03 and cond04 and cond05
        condition = condition and cond06 and cond07 and cond08 and cond09
        condition = condition and cond10 and cond11 and cond12
        if not condition:
            raise ValueError

        # initial definitions
        self.borders = borders
        self.materials = range(len(materials))
        self.boundaries = boundaries
        self.amb_temperature = amb_temperature

        # loads the data for each material
        for i in range(len(materials)):
            tadi = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'tadi.txt'
            tadd = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'tadd.txt'
            cpa = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'cpa.txt'
            cp0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'cp0.txt'
            k0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'k0.txt'
            ka = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'ka.txt'
            rho0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'rho0.txt'
            rhoa = os.path.dirname(os.path.realpath(__file__)) + \
                '/../database/' + materials[i] + '/' + 'rhoa.txt'
            self.materials[i] = mats.calmatpro(
                tadi, tadd, cpa, cp0, k0, ka, rho0, rhoa)

        # defines which are the properties of each material point
        self.materials_index = [None]
        for i in range(len(borders) - 1):
            for j in range(borders[i], borders[i + 1]):
                self.materials_index.append(materials_order[i])
        self.materials_index.append(None)

        # loads the inputs and creates the lists temperature, Cp, rho and k
        self.num_points = borders[-1] + 1
        self.file_name = file_name
        self.dx = dx
        self.dt = dt
        self.temperature = [[amb_temperature, amb_temperature]]
        self.Cp = [None]
        self.rho = [None]
        self.Q = [None]
        self.Q0 = [None]
        self.k = [self.materials[self.materials_index[1]].k0(amb_temperature)]
        for i in range(1, borders[-1]):
            self.temperature.append([amb_temperature, amb_temperature])
            self.rho.append(self.materials[self.materials_index[i]].rho0(
                amb_temperature))
            self.Cp.append(
                self.materials[self.materials_index[i]].cp0(amb_temperature))
            self.k.append(
                self.materials[self.materials_index[i]].k0(amb_temperature))
            self.Q.append(0.)
            self.Q0.append(0.)
        self.temperature.append([amb_temperature, amb_temperature])
        self.rho.append(None)
        self.Cp.append(None)
        self.Q.append(None)
        self.Q0.append(None)
        self.k.append(
            self.materials[self.materials_index[-2]].k0(amb_temperature))

        if Q != []:
            self.Q = Q
        if Q0 != []:
            self.Q0 = Q0

        # initializes the state of each point (True if active and False if ...
        # unactive), the time, heat flux from the hot side and heat flux ...
        # from the cold side
        self.state = [initial_state for i in range(borders[-1] + 1)]
        self.time_passed = 0.

        self.Q_ref = copy.copy(self.Q)
        self.Q0_ref = copy.copy(self.Q0)

        line = 'time(s)'
        for i in range(len(self.temperature)):
            line = line + ',T[' + str(i) + '] (K)'
        if heat_save:
            line = line + ',Q (J/m)'
        line = line + '\n'
        f = open(self.file_name, 'a')
        f.write(line)
        f.close()

    def activate(self, initial_point, final_point):
        """Activation of the material

        activates a given space interval of the material,
        between the initial_point and final_point.

        """

        for i in range(initial_point, final_point):

            if self.state[i] is False:
                self.temperature[i][0] = self.temperature[i][0] + \
                    self.materials[self.materials_index[i]].tadi(
                        self.temperature[i][0])
                self.rho[i] = self.materials[self.materials_index[i]].rhoa(
                    self.temperature[i][0])
                self.Cp[i] = self.materials[self.materials_index[i]].cpa(
                    self.temperature[i][0])
                self.k[i] = self.materials[self.materials_index[i]].ka(
                    self.temperature[i][0])
                self.state[i] = True

            else:
                message = 'point %f already activated' % float(i)
                print message

    def deactivate(self, initial_point, final_point):
        """Deactivation of the material

        deactivates a given space interval of the material,
        between the initial_point and final_point.

        """

        for i in range(initial_point, final_point):

            if self.state[i] is True:
                self.temperature[i][0] = self.temperature[i][0] - \
                    self.materials[self.materials_index[i]].tadd(
                        self.temperature[i][0])
                self.rho[i] = self.materials[self.materials_index[i]].rho0(
                    self.temperature[i][0])
                self.Cp[i] = self.materials[self.materials_index[i]].cp0(
                    self.temperature[i][0])
                self.k[i] = self.materials[self.materials_index[i]].k0(
                    self.temperature[i][0])
                self.state[i] = False

            else:
                message = 'point %f already deactivated' % float(i)
                print message



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the explicit_k solver.

Used to compute thermal processes

"""

import numpy as np
import copy


def explicit_k(obj):
    """explicit_k solver.

    Used to compute one time step of systems with x-dependent thermal
    contuctivity.

    """

    x = copy.copy(obj.temperature)

    # computes
    for i in range(1, obj.num_points - 1):

        eta = obj.dt / (2. * obj.rho[i] * obj.Cp[i] * obj.dx * obj.dx)
        beta = obj.dt / (obj.rho[i] * obj.Cp[i])

        Tnew = ((1 + beta * obj.Q[i]) * obj.temperature[i][0] +
                eta * ((obj.k[i + 1] + obj.k[i]) * obj.temperature[i + 1][0] -
                       (obj.k[i - 1] + obj.k[i + 1] + 2 * obj.k[i]) *
                       obj.temperature[i][0] + (obj.k[i - 1] + obj.k[i]) *
                       obj.temperature[i - 1][0]) +
                beta * (obj.Q0[i] - obj.Q[i] * obj.amb_temperature))

        x[i][1] = Tnew

    # left boundary for next time step
    if obj.boundaries[0] == 0:
        x[0][1] = obj.temperature[1][1]
    else:
        x[0][1] = obj.boundaries[0]

    # right boundary for next time step
    if obj.boundaries[1] == 0:
        x[obj.num_points - 1][1] = obj.temperature[obj.num_points - 2][1]
    else:
        x[obj.num_points - 1][1] = obj.boundaries[1]

    return x



"""Contains the implicit_k solver.

Used to compute thermal processes

"""

import numpy as np
import copy


def implicit_k(obj):
    """implicit_k solver.

    Used to compute one time step of systems with k-dependent thermal
    contuctivity.

    """

    # initializes the matrixes for the equation systems
    a = np.zeros((obj.num_points, obj.num_points))
    b = np.zeros(obj.num_points)

    # left boundary
    a[0][0] = 1
    if obj.boundaries[0] == 0:
        b[0] = obj.temperature[1][0]
    else:
        b[0] = obj.boundaries[0]

    # right boundary
    a[obj.num_points - 1][obj.num_points - 1] = 1
    if obj.boundaries[1] == 0:
        value = obj.temperature[obj.num_points - 2][0]
        b[obj.num_points - 1] = value
    else:
        b[obj.num_points - 1] = obj.boundaries[1]

    # creates the matrixes and solves the equation systems
    for i in range(1, obj.num_points - 1):
        gamma = 4. * obj.rho[i] * obj.Cp[i] * obj.dx * obj.dx / obj.dt

        a[i][i - 1] = obj.k[i - 1] + obj.k[i]
        a[i][i] = -(gamma + obj.k[i + 1] + obj.k[i - 1] +
                    2. * obj.k[i] - 2 * obj.dt * obj.dt *
                    obj.Q[i])
        a[i][i + 1] = obj.k[i + 1] + obj.k[i]
        b[i] = -(obj.k[i + 1] + obj.k[i]) * \
            obj.temperature[i + 1][0] + \
            (-gamma + obj.k[i + 1] + obj.k[i - 1] + 2. *
                obj.k[i] - 2 * obj.dt * obj.dt * obj.Q[i]) * \
            obj.temperature[i][0] - \
            (obj.k[i - 1] + obj.k[i]) * \
            obj.temperature[i - 1][0] - \
            4. * obj.dx * obj.dx * \
            (obj.Q0[i] - obj.Q[i] * obj.amb_temperature)

    x = np.linalg.solve(a, b)

    y = copy.copy(obj.temperature)

    for i in range(obj.num_points):
        y[i][1] = x[i]
        y[i][0] = x[i]

    return y



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the implicit_general solver.

Used to compute thermal processes

"""

import numpy as np
import copy


def implicit_general(obj):
    """implicit_general solver.

    Used to compute one time step of systems with fixed thermal contuctivity.

    """

    # initializes the matrixes for the equation systems
    a = np.zeros((obj.num_points, obj.num_points))
    b = np.zeros(obj.num_points)

    # left boundary
    a[0][0] = 1
    if obj.boundaries[0] == 0:
        b[0] = obj.temperature[1][0]
    else:
        b[0] = obj.boundaries[0]

    # right boundary
    a[obj.num_points - 1][obj.num_points - 1] = 1
    if obj.boundaries[1] == 0:
        value = obj.temperature[obj.num_points - 2][0]
        b[obj.num_points - 1] = value
    else:
        b[obj.num_points - 1] = obj.boundaries[1]

    # creates the matrixes and solves the equation systems
    for i in range(1, obj.num_points - 1):
        beta = obj.k[i] * obj.dt / \
            (2 * obj.rho[i] * obj.Cp[i] * obj.dx * obj.dx)
        sigma = obj.dt / (obj.rho[i] * obj.Cp[i])

        a[i][i - 1] = -beta
        a[i][i] = 1 + 2 * beta - sigma * obj.Q[i]
        a[i][i + 1] = -beta
        b[i] = (1 - 2 * beta - sigma * obj.Q[i]) * \
            obj.temperature[i][0] + \
            beta * obj.temperature[i + 1][0] + \
            beta * obj.temperature[i - 1][0] + 2. * sigma * \
            (obj.Q0[i] - obj.Q[i] * obj.amb_temperature)

    x = np.linalg.solve(a, b)
    y = copy.copy(obj.temperature)

    # updates the temperature list
    for i in range(obj.num_points):
        y[i][1] = x[i]
        y[i][0] = x[i]

    return y



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the class calmatpro.

Used to access the properties of materials from the database

"""

import numpy as np


class calmatpro:

    """Calmatpro class

    Loads and gives physical properties of materials from the database

    """

    def __init__(self, nameTadi, nameTadd, nameCpA, nameCp0, namek0, namekA,
                 namerho0, namerhoA):
        """Initializes the object.

        loads the physical properties of a materials
        nameTadi is the adiabatic temperature increase file
        nameTadd is the adiabatic temperature decrease file
        nameCp0 is the specific heat file
        nameCpA is the active specific heat file
        namek0 is the thermal conductivity file
        namekA is the active thermal conductivity file
        namerho0 is the density file
        namerhoA is the active density file

        """

        # adiabatic temperature increase
        input = open(nameTadi, 'r')
        s = input.readlines()
        self.xTadi = []
        self.yTadi = []
        for line in s:
            pairTad = line.split()
            self.xTadi.append(float(pairTad[0]))
            self.yTadi.append(float(pairTad[1]))
        input.close()

        # adiabatic temperature decrease
        input = open(nameTadd, 'r')
        s = input.readlines()
        self.xTadd = []
        self.yTadd = []
        for line in s:
            pairTad = line.split()
            self.xTadd.append(float(pairTad[0]))
            self.yTadd.append(float(pairTad[1]))
        input.close()

        # specific heat
        input = open(nameCpA, 'r')
        s = input.readlines()
        self.xCpA = []
        self.yCpA = []
        for line in s:
            pairTad = line.split()
            self.xCpA.append(float(pairTad[0]))
            self.yCpA.append(float(pairTad[1]))
        input.close()

        # active specific heat
        input = open(nameCp0, 'r')
        s = input.readlines()
        self.xCp0 = []
        self.yCp0 = []
        for line in s:
            pairTad = line.split()
            self.xCp0.append(float(pairTad[0]))
            self.yCp0.append(float(pairTad[1]))
        input.close()

        # thermal conductivity
        input = open(namek0, 'r')
        s = input.readlines()
        self.xk0 = []
        self.yk0 = []
        for line in s:
            pairTad = line.split()
            self.xk0.append(float(pairTad[0]))
            self.yk0.append(float(pairTad[1]))
        input.close()

        # active thermal conductivity
        input = open(namekA, 'r')
        s = input.readlines()
        self.xkA = []
        self.ykA = []
        for line in s:
            pairTad = line.split()
            self.xkA.append(float(pairTad[0]))
            self.ykA.append(float(pairTad[1]))
        input.close()

        # density
        input = open(namerho0, 'r')
        s = input.readlines()
        self.xrho0 = []
        self.yrho0 = []
        for line in s:
            pairTad = line.split()
            self.xrho0.append(float(pairTad[0]))
            self.yrho0.append(float(pairTad[1]))
        input.close()

        # active density
        input = open(namerhoA, 'r')
        s = input.readlines()
        self.xrhoA = []
        self.yrhoA = []
        for line in s:
            pairTad = line.split()
            self.xrhoA.append(float(pairTad[0]))
            self.yrhoA.append(float(pairTad[1]))
        input.close()

    def tadi(self, Temperature):
        """gives the adiabatic temperature increase for a given temperature"""

        return np.interp(Temperature, self.xTadi, self.yTadi)

    def tadd(self, Temperature):
        """gives the adiabatic temperature decrease for a given temperature"""

        return np.interp(Temperature, self.xTadd, self.yTadd)

    def cpa(self, Temperature):
        """gives the active specific heat for a given temperature"""

        return np.interp(Temperature, self.xCpA, self.yCpA)

    def cp0(self, Temperature):
        """gives the specific heat for a given temperature"""

        return np.interp(Temperature, self.xCp0, self.yCp0)

    def k0(self, Temperature):
        """gives the thermal conductivity for a given temperature"""

        return np.interp(Temperature, self.xk0, self.yk0)

    def ka(self, Temperature):
        """gives the active thermal conductivity for a given temperature"""

        return np.interp(Temperature, self.xkA, self.ykA)

    def rho0(self, Temperature):
        """method gives the density for a given temperature"""

        return np.interp(Temperature, self.xrho0, self.yrho0)

    def rhoa(self, Temperature):
        """method gives the active density for a given temperature"""

        return np.interp(Temperature, self.xrhoA, self.yrhoA)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Systems.

Systems

"""

from solid_caloric_systems import solid_active_regenerator
from fluid_caloric_systems import fluid_active_regenerator
